import * as React from 'react';
import {useEffect, useState} from 'react';
import './style.css';

// import { styled } from '@mui/material/styles';
import Grid from '@mui/material/Grid2';

import AddIcon from '@mui/icons-material/Add';

import Box from '@mui/material/Box';
import TextField from '@mui/material/TextField';
import Stack from '@mui/material/Stack';
import Button from '@mui/material/Button';
// import Rating from '@mui/material/Rating';
// import Typography from '@mui/material/Typography';
// import StarIcon from '@mui/icons-material/Star';
import ReplySharpIcon from '@mui/icons-material/ReplySharp';

import { Select, MenuItem, InputLabel, FormHelperText } from '@mui/material';

import Alert from '@mui/material/Alert';

import Swal from 'sweetalert2'
import withReactContent from 'sweetalert2-react-content'

import axios from 'axios';
import { useAuth } from "../../hooks/AuthProvider";

import { useNavigate } from 'react-router-dom';

import Nav from '../nav';
import Config from '../config';

const withRouter = (Component) => {
  const Wrapper = (props) => {
    const navigate = useNavigate();
    return <Component navigate={navigate} {...props} />;
  };
  return Wrapper;
};

function CompraAdd(props) {

  const [rows, setRows] = useState([
    {
      "idct_producto": 0,
      "ct_grupo_idct_grupo": 0,
      "Nombre": "",
      "Cantidad": 0,
      "ct_unidad_idct_unidad": 0,
      "unidad": "",
      "Foto": "",
      "FechaUltCambio": "",
      "UsuarioUltCambio": 0
    }
  ]);

  const [unidad, setUnidad] = useState([
    {
      "idct_unidad": 0,
      "Titulo": ""
    }
  ]);


  const [refresh, setRefresh] = useState(true);

  const auth = useAuth();
  
  const [validacionForms, setValidacionForms] = useState(true);
  const [errorForms, setErrorForms] = useState('');

  const [input, setInput] = useState({
    producto: "",
    nombreProducto: "",
    cantidad: "",
    unidad: ""
  });

  const [errorProducto, setErrorProducto] = useState(false);
  const [helperProducto, setHelperProducto] = useState('');

  const [errorCantidad, setErrorCantidad] = useState(false);
  const [helperCantidad, setHelperCantidad] = useState('');

  const [errorUnidad, setErrorUnidad] = useState(false);
  const [helperUnidad, setHelperUnidad] = useState('');

  // var formWitdh = window.innerWidth;
  // console.log("Ancho de pantalla: ",formWitdh);
  
  // const auth = useAuth();
  // console.log("Función AUTH: ",auth);

  // Notificación error grave
    function errorNotification(code, message) {

      const MySwal = withReactContent(Swal); 
      MySwal.fire({
        icon: 'error',
        title: message,
        text: '',
        timer: 5000,
        timerProgressBar: true,
        toast: true,
        position: "center",
        footer: ''
      })
      props.navigate(Config.URL_APP+'/logout');
    }

    // Cargar los datos del Server cuando se carga y recarga la página
    const fetchProductos = async () => {
      // console.log("Valor de la variable Refresh: ",refresh);
      if (refresh) {        // Para intentar que sólo sea una vez
        // setRefresh(false);
        const url = Config.RUTA_API + "/listadoProductos"; 
        const formData = new FormData();
        const config = {
          headers: {
              "Content-Type": "application/x-www-form-urlencoded",
              "token-user": auth.token
          },
        };
        try {  
          const response = await axios.post(url,formData,config)     
              var data = response.data;
              // console.log("Respuesta del fetch: ",data);
              setRows(data);
  
              if (data.error === true )  { 
                console.log("Hay un error: ",data.message);
                errorNotification(data.error,data.message);
              } else {
                // console.log("Registros de respuesta: ", data.length);
              }
                
            }
        catch(error) {
              console.log("Error get Listados Producto", error);
              errorNotification('000',error);
            }
      }
    };

// Cargar los datos del Server cuando se carga y recarga la página
const fetchUnidades = async () => {
  // console.log("Valor de la variable Refresh: ",refresh);
  if (refresh) {        // Para intentar que sólo sea una vez
    setRefresh(false);
    const url = Config.RUTA_API + "/listadoUnidades"; 
    const formData = new FormData();
    const config = {
      headers: {
          "Content-Type": "application/x-www-form-urlencoded",
          "token-user": auth.token
      },
    };
    try {  
      const response = await axios.post(url,formData,config)     
          var data = response.data;
          // console.log("Respuesta del fetch: ",data);
          setUnidad(data);

          if (data.error === true )  { 
            console.log("Hay un error: ",data.message);
            errorNotification(data.error,data.message);
          } else {
            // console.log("Registros de respuesta: ", data.length);
          }
            
        }
    catch(error) {
          console.log("Error get Listados Producto", error);
          errorNotification('000',error);
        }
  }
};
    
  
    // Cargar los datos del Server cuando se carga la página
    useEffect(() => {
      console.log("Ejecutándose 'useEffect");
      fetchProductos();
      fetchUnidades();
       }, [refresh]);

  // Envío del Formulario
  const saveForms = (e) => { 
    e.preventDefault();
    setValidacionForms(true); // Quitamos errores del formulario
    setErrorForms('');

    var existeError = false;
    var Error = '';
  
    if (input.producto === '') {
      setHelperProducto('Hay que seleccionar un Producto');
      setErrorProducto(true);
      existeError = true;
      Error += ' (Producto) ';
    }
    if (input.cantidad === '') {
      setHelperCantidad('Hay indicar una Cantidad');
      setErrorCantidad(true);
      existeError = true;
      Error += ' (Cantidad) ';
    }
    if (input.unidad === '') {
      setHelperUnidad('Hay que informar de las unidades o medidas');
      setErrorUnidad(true);
      existeError = true;
      Error += ' (Unidad) ';
    }


    if (!(existeError)) { // se han introducido los valores de los campos
      // console.log("accediendo al Server");
      // Acceso al server y status de la operación
      const url = Config.RUTA_API + "/compra/add/0";
      const formData = new FormData();
        formData.append('idct_producto', input.producto);
        formData.append('cantidad', input.cantidad);
        formData.append('unidad', input.unidad);
      const config = {
        headers: {
            "Content-Type": "application/x-www-form-urlencoded",
            "token-user": auth.token
        },
      };
      
      axios.post(url, formData, config)
  
        .then((response) => {
          const res = response.data;
          // console.log("Respuesta del fetch: ",res);
          if (res.error === false) {
            
            setValidacionForms(true); // Quitamos errores del formulario
            setErrorForms('');
            postSave();               // Alert y navegar a la aplicación
  
          } else {
            setValidacionForms(false); 
            setErrorForms(res.message);  
          }
        })
        .catch((error) => {
          console.error("Error send Register: ", error);
          setValidacionForms(false); 
          setErrorForms("Error send Register: ", error); 
          errorNotification('000', error)

        });
  
    } else {
    
      setValidacionForms(false); // Ponemos los errores del formulario
      
      Error = 'Falta la entrada de los datos o la información de estos campos está mal: '+ Error;
      setErrorForms(Error);
    }
  };

  const postSave = (e) => {

    const MySwal = withReactContent(Swal); 
    MySwal.fire({
      icon: 'info',
      title: 'Nuevo Producto creado!!!!',
      text: '',
      timer: 1000,
      timerProgressBar: true,
      toast: true,
      position: "center",
      footer: ''
     })
    props.navigate(Config.URL_APP+'/compraList');
  }

  const handleValidationProducto = (e) => {

    const { name, value } = e.target;

    const row = rows.find((rowid) => rowid.idct_producto === value);
    // console.log("Datos del Producto: ",row);

    setInput((prev) => (
    { ...prev,
      producto: row.idct_producto,
      nombreProducto: row.Nombre,
      cantidad: row.Cantidad,
      unidad: row.ct_unidad_idct_unidad
    }));

    // console.log("Datos de Input: ",input);


  };

  const handleValidationCantidad = (e) => {

    const { name, value } = e.target;
    setInput((prev) => ({
      ...prev,
      [name]: value,
    }));
    // const reg = new RegExp("^[0-9]{1,10}(.[0-9]{1,3})?$");
    const reg = new RegExp("^[0-9]{1,10}$");
    if (! reg.test(e.target.value)) {
      setHelperCantidad('Entrada inválida para "Cantidad"');
    } else {
      setHelperCantidad('');
    }
    setErrorCantidad(!reg.test(e.target.value));
  };

  const handleValidationUnidad = (e) => {

    const { name, value } = e.target;
    setInput((prev) => ({
      ...prev,
      [name]: value,
    }));
    
  };


  return (
    <>
    <Nav /> 
    <div className='titlePage'>
        <h3 className="head_title">Añadir artículo a la lista</h3>
    </div>


    <div className='panel-compraAdd'>
      <div className='box-compraAdd'>

        <Box component="form" noValidate autoComplete="off" >  
          <Grid container spacing={2}>
          <Grid xs={12}>
                <InputLabel id="select-label">Selecciona el Producto *</InputLabel>
                <Select 
                  error={errorProducto} 
                  id="producto" 
                  name="producto"
                  label="Producto" 
                  required={false}
                  variant="standard"   
                  className= 'CustomField'  
                  value={input.producto}
                  onChange={handleValidationProducto}
                >
                {rows.map((row, index) => (
                  <MenuItem key={row.idct_producto}  value={row.idct_producto}> {row.Nombre} 
                  </MenuItem> 
                ))}
                </Select>
                {errorProducto && <FormHelperText>{helperProducto}</FormHelperText>}
            </Grid>

            <Grid xs={12} >
                <TextField  
                  error={errorCantidad} 
                  required={true}
                  id="cantidad"
                  name="cantidad" 
                  label="Cantidad" 
                  variant="standard"   
                  type= 'text' 
                  className= 'CustomField'  
                  helperText={helperCantidad}
                  value={input.cantidad}
                  onChange={(e) => handleValidationCantidad(e)}
                  />
            </Grid>
            <Grid xs={12}>
                <InputLabel id="select-label">Unidad/Medida *</InputLabel>
                <Select 
                  error={errorUnidad} 
                  id="unidad" 
                  name="unidad"
                  label="Unidad" 
                  required={true}
                  variant="standard"   
                  className= 'CustomField'  
                  value={input.unidad}
                  onChange={handleValidationUnidad}
                >
                {unidad.map((row, index) => (
                  <MenuItem key={row.idct_unidad}  value={row.idct_unidad}> {row.Titulo} 
                  </MenuItem> 
                ))}
                </Select>
                {errorUnidad && <FormHelperText>{helperUnidad}</FormHelperText>}
            </Grid>
          </Grid>
          <Stack spacing={2} direction="row" sx={{ width: '100%', padding: '15px 10px 15px 10px'}} >
              <Button variant="contained" type='submit'  onClick={(e) => saveForms(e)}>Guardar</Button>
              <Button variant="outlined"  startIcon={<ReplySharpIcon />}  onClick={(e) =>props.navigate(Config.URL_APP+'/compraList')}>Volver</Button> 
              <Button variant="outlined"  startIcon={<AddIcon />} onClick={(e) =>props.navigate(Config.URL_APP+'/productoList')}>Producto</Button>
          </Stack>
          {
          <Stack sx={{ width: '100%' }} spacing={2}>
            { validacionForms? '': <Alert severity="error">{errorForms}</Alert>}
          </Stack>

          }
        </Box>
      </div>
    </div>
    </>
  );
}

// IMPORTANT ----------------------------------------
export default withRouter( CompraAdd);