import * as React from 'react';
import {useEffect, useState} from 'react';
import PropTypes from 'prop-types';
import AppBar from '@mui/material/AppBar';
import Box from '@mui/material/Box';
import CssBaseline from '@mui/material/CssBaseline';
import Divider from '@mui/material/Divider';
import Drawer from '@mui/material/Drawer';
import IconButton from '@mui/material/IconButton';
import List from '@mui/material/List';
import ListItem from '@mui/material/ListItem';
import ListItemButton from '@mui/material/ListItemButton';
import ListItemText from '@mui/material/ListItemText';
import MenuIcon from '@mui/icons-material/Menu';
import Toolbar from '@mui/material/Toolbar';
import Typography from '@mui/material/Typography';
import Button from '@mui/material/Button';
import { useNavigate } from 'react-router-dom';

import PersonIcon from '@mui/icons-material/Person4';

import Config from '../config';
import Logo from '../logo';
import { useAuth } from "../../hooks/AuthProvider";

const withRouter = (Component) => {
    const Wrapper = (props) => {
      const navigate = useNavigate();
      return <Component navigate={navigate} {...props} />;
    };
    return Wrapper;
  };

// Opciones de usuario identificado
const navItems1 = [ 
    'Lista',
    'Productos',
    'Grupos',
    'Usuarios',
    'Select. Grupo',
    'UserConnect'
  ];

const link1 = [
    '/compraList',
    '/productoList',
    '/grupoList',
    '/userList',
    '/selectgroup',
    '/userconnect'
]

// Opciones de usuario SIN identificación
const navItems2 = [
  'Identificación',
  'Nuevo Usuario',
 // 'perdida de password',
];

const link2 = [
  '/login',
  '/register'
 //  '/newPassword',
]

const drawerWidth = 240;

function Nav(props) {
  const { window } = props;
  const [mobileOpen, setMobileOpen] = useState(false);

  const [navItems, setnavItems] = useState([]);
  const [link, setlink] = useState([]);

  const auth = useAuth();

  
  // Cargar los datos del Server cuando se carga la página
  useEffect(() => {
    // console.log("Ejecutando 'useEffect() en Nav", auth);
    const token = auth.token;
    const groupName = auth.groupName;
    if (token ==='' || groupName === null ) {
      setnavItems(navItems2);
      setlink(link2);
    } else {
      setnavItems(navItems1);
      setlink(link1);
    }
  }, [auth.token,auth.groupName ]);
  
  
  const handleDrawerToggle = () => {
    setMobileOpen((prevState) => !prevState);
  };


  const drawer = (   // Menú para el movil
    <Box onClick={handleDrawerToggle} sx={{ textAlign: 'center' }}>
      
      <Typography variant="h6" sx={{ my: 2 }}>
      <Logo /> {Config.TITLE_APP} 
      </Typography>
      
      <Divider />
      <List>
        {navItems.map((item, index) => (
          item === 'UserConnect'?
          <ListItem key={item} disablePadding>
            <ListItemButton>
                <IconButton onClick={(event) => handleMenuItemClick(event, index)}>
                  <PersonIcon fontSize='large'/>
                </IconButton>
            </ListItemButton>
          </ListItem>
        :
          <ListItem key={item} disablePadding>
            <ListItemButton sx={{ textAlign: 'left' }} onClick={(event) => handleMenuItemClick(event, index)}>
              <ListItemText primary={item} />
            </ListItemButton>
          </ListItem>
        ))}
      </List>
    </Box>
  );

  const container = window !== undefined ? () => window().document.body : undefined;
  
  const handleMenuItemClick = (event, index) => {
    props.navigate(Config.URL_APP+link[index]);
  };

  return (
    <Box sx={{ display: 'flex' }}>
      <CssBaseline />
      <AppBar component="nav">
        <Toolbar>
          <IconButton
            color="inherit"
            aria-label="open drawer"
            edge="start"
            onClick={handleDrawerToggle}
            sx={{ mr: 2, display: { sm: 'none' } }}
          >
            <MenuIcon /> <Logo /> {Config.TITLE_APP}
          </IconButton>
          <Typography
            variant="h6"
            component="div"
            sx={{ flexGrow: 1, display: { xs: 'none', sm: 'block' } }}
          >
            <Logo /> {Config.TITLE_APP}
          </Typography>
          <Box sx={{ display: { xs: 'none', sm: 'block' } }}>
            {navItems.map((item, index) => (
              item === 'UserConnect'?
                <IconButton key={item}  sx={{ color: '#fff' }} onClick={(event) => handleMenuItemClick(event, index)} >
                  <PersonIcon fontSize='large' />
                </IconButton>
              :
              <Button key={item} sx={{ color: '#fff' }} onClick={(event) => handleMenuItemClick(event, index)}>
                {item}
              </Button>
              
            ))}
          </Box> 
        </Toolbar>
      </AppBar>
      <nav>
        <Drawer
          container={container}
          variant="temporary"
          open={mobileOpen}
          onClose={handleDrawerToggle}
          ModalProps={{
            keepMounted: true, // Better open performance on mobile.
          }}
          sx={{
            display: { xs: 'block', sm: 'none' },
            '& .MuiDrawer-paper': { boxSizing: 'border-box', width: drawerWidth },
          }}
        >
          {drawer}
        </Drawer>
      </nav>
    
      <Box component="main" sx={{ p: 1 }}>
        {/* <Box component="main" sx={{ p: 3 }}> */}
        <Toolbar />
        {/*
        <Typography>
         
          Lorem ipsum dolor sit amet consectetur adipisicing elit. Similique unde
          fugit veniam eius, perspiciatis sunt? Corporis qui ducimus quibusdam,
          aliquam dolore excepturi quae. Distinctio enim at eligendi perferendis in
          cum quibusdam sed quae, accusantium et aperiam? Quod itaque exercitationem,
          at ab sequi qui modi delectus quia corrupti alias distinctio nostrum.
          
        </Typography>
        */}
        {/* </Box> */}
      </Box>

    </Box>
  );
}

Nav.propTypes = {
  /**
   * Injected by the documentation to work in an iframe.
   * You won't need it on your project.
   */
  window: PropTypes.func,
};

// IMPORTANT ----------------------------------------
export default withRouter( Nav );
